import { IconName } from "@fortawesome/fontawesome-svg-core";
import Box from "@mui/material/Box";
import Button from "@mui/material/Button";
import Dialog from "@mui/material/Dialog";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";
import DialogContentText from "@mui/material/DialogContentText";
import DialogTitle from "@mui/material/DialogTitle";
import Stack from "@mui/material/Stack";
import React from "react";
import {
  colorSwitch,
  NerdIcon,
  setCustomIconDefinition,
} from "../../shared/NerdIcon";
import BorderLinearProgress from "../../shared/Progess/BorderLinearProgress";
import { NerdNotificationProps } from "../typings";
import "../NerdNotifier.scss";

type openHookAction = {
  open: boolean;
  onHandleClose: (evt: boolean) => void;
};
/**
 *
 * @param {NerdNotificationProps & openHookAction} param0 Component props
 * @returns {JSX.Element} Intrusive nerdNotifier
 */
export function Intrusive({
  title,
  subtitle,
  body,
  icon,
  actions,
  severity = "info",
  open,
  iconColor,
  autoHideDuration = 5000,
  autohide = true,
  showIcon = true,
  onHandleClose,
  unstyled = true,
  actionPrimaryProps = {
    variant: "text",
    color: "primary",
  },
  actionSecondaryProps = {
    variant: "text",
    color: "inherit",
  },
}: NerdNotificationProps & openHookAction): JSX.Element {
  const [progress, setProgress] = React.useState(0);
  const timer = React.useRef<number>(0);
  const handleClose = (_event?: unknown, reason?: string) => {
    if (
      reason === "clickaway" ||
      reason === "escapeKeyDown" ||
      reason === "backdropClick"
    ) {
      return;
    }
    onHandleClose(false);
  };

  React.useEffect(() => {
    timer.current = window.setInterval(() => {
      setProgress((prevProgress) =>
        prevProgress >= 100 ? 0 : prevProgress + 10
      );
    }, autoHideDuration / 10);

    return () => {
      clearInterval(timer.current);
    };
  }, []);

  React.useEffect(() => {
    if (progress >= 100 && autohide) {
      onHandleClose(false);
    }
  }, [progress, autohide, open]);

  const severityIcon = setCustomIconDefinition(
    (severityIcons(severity) ?? "exclamation-circle") as IconName
  );

  return (
    <Dialog
      open={open}
      onClose={handleClose}
      aria-labelledby="alert-dialog-title"
      aria-describedby="alert-dialog-description"
      keepMounted
      transitionDuration={1000}
      classes={{
        paper: "NerdNotifierIntrusivePaper",
      }}
    >
      <Stack spacing={1}>
        {autohide && !actions && (
          <BorderLinearProgress
            variant="determinate"
            value={progress}
            color={severity}
          />
        )}
        {showIcon && (
          <Box className="IntrusiveIcon">
            {typeof icon === "string" ? (
              <NerdIcon
                icon={severityIcon}
                severity={severity}
                iconColor={iconColor}
              />
            ) : (
              icon
            )}
            {severity && !icon && (
              <NerdIcon
                icon={severityIcon}
                severity={severity}
                iconColor={colorSwitch(severity)}
              />
            )}
          </Box>
        )}
        {title && <DialogTitle id="alert-dialog-title">{title}</DialogTitle>}
        {unstyled ? (
          <DialogContent>
            {subtitle && (
              <DialogContentText id="alert-dialog-description">
                {subtitle}
              </DialogContentText>
            )}
            {body}
          </DialogContent>
        ) : (
          <React.Fragment>
            {subtitle}
            {body}
          </React.Fragment>
        )}
        {!autohide && (
          <DialogActions>
            <Button onClick={handleClose} {...actionSecondaryProps}>
              Cancel
            </Button>
            <Button onClick={handleClose} {...actionPrimaryProps}>
              Ok
            </Button>
          </DialogActions>
        )}
      </Stack>
    </Dialog>
  );
}

/**
 *
 * @param severityText
 */
function severityIcons(severityText: string): string {
  switch (severityText) {
    case "error":
      return "times";
    case "info":
      return "exclamation";
    case "success":
      return "check";
    case "warning":
      return "exclamation-triangle";
    default:
      return "";
  }
}
