import React from "react";
import MuiAlert, { AlertProps } from "@mui/material/Alert";
import { NerdNotificationProps } from "../typings";
import Slide, { SlideProps } from "@mui/material/Slide";
import Snackbar from "@mui/material/Snackbar";
import { Stack, Typography } from "@mui/material";
import Box from "@mui/system/Box";
import BorderLinearProgress from "../../shared/Progess/BorderLinearProgress";

type TransitionProps = Omit<SlideProps, "direction">;

type openHookAction = {
  open: boolean;
  handleClose: (event?: React.SyntheticEvent | Event, reason?: string) => void;
};

let direction: "up" | "down" | "left" | "right" | undefined = undefined;

/**
 *
 * @param {NerdNotificationProps & openHookAction} param0 components props
 * @returns {JSX.Element} NonIntrusive nerdNotifier
 */
export function NonIntrusive({
  uuid,
  title,
  transition = "up",
  severity = "info",
  subtitle,
  body,
  position = {
    vertical: "bottom",
    horizontal: "right",
  },
  actions,
  autoHideDuration = 5000,
  autohide = true,
  open,
  handleClose,
  icon,
  color,
}: NerdNotificationProps & openHookAction): JSX.Element {
  direction = transition;
  const [progress, setProgress] = React.useState(0);
  const timer = React.useRef<number>(0);

  React.useEffect(() => {
    timer.current = window.setInterval(() => {
      setProgress((prevProgress) =>
        prevProgress >= 100 ? 0 : prevProgress + 10
      );
    }, autoHideDuration / 10);
    return () => {
      clearInterval(timer.current);
    };
  }, []);

  React.useEffect(() => {
    if (progress >= 100 && autohide) {
      handleClose();
    }
  }, [progress, autohide]);

  if (icon) {
    return (
      <Alert severity="warning" className="nonIntrusive--unkown">
        Unkown
      </Alert>
    );
  }

  return (
    <Box component="div" id={uuid}>
      <Snackbar
        open={open}
        autoHideDuration={autohide ? autoHideDuration : null}
        TransitionComponent={Transition}
        anchorOrigin={position}
      >
        <Stack spacing={1}>
          {autohide && (
            <BorderLinearProgress
              variant="determinate"
              value={progress}
              color={severity}
            />
          )}
          <Alert
            severity={severity}
            sx={{ width: "100%", alignItems: "center", backgroundColor: color }}
            action={actions}
            onClose={handleClose}
          >
            {title && <Typography variant="body1">{title}</Typography>}
            {subtitle && <Typography variant="body2">{subtitle}</Typography>}
            {body && <Typography variant="caption">{body}</Typography>}
          </Alert>
        </Stack>
      </Snackbar>
    </Box>
  );
}

const Alert = React.forwardRef<HTMLDivElement, AlertProps>(function Alert(
  props,
  ref
) {
  return <MuiAlert ref={ref} variant="filled" {...props} />;
});

/**
 *
 * @param {TransitionProps} props Component props
 * @returns {JSX.Element} Transition Component
 */
function Transition(props: TransitionProps): JSX.Element {
  return <Slide {...props} direction={direction} />;
}
